
%%Scenario generation script

%%
%Read data
Irradiancefile = 'irradiance_rissa.xlsx';
Loadfile = 'last_sorburoy.xlsx';
Windfile = 'Forbruk_og_vind.xlsx';

irradiance = xlsread(Irradiancefile,'C2:C43801');  %5 years of irradiance data in one vector [43800x1]
loadvector = xlsread(Loadfile,'I2:I8761');  %One year of historic load data
Wind = xlsread(Windfile,'Sheet1','C9:G8768'); %Actual measured wind for 4 years

windvec = reshape(Wind,43800,1);


%%
%Calculates 4 first moments for each complete dataset
windmom = [mean(windvec), var(windvec),skewness(windvec),kurtosis(windvec)];
irrmom = [mean(irradiance), var(irradiance),skewness(irradiance),kurtosis(irradiance)];

monthhour = [744,672,744,720,744,720,744,744,720,744,720,744];

%%
%Monthize function groups 5 years of historic data into months
windmonth = monthize(Wind,monthhour);
irrmonth = monthize(irradiance,monthhour);
loadmonth = monthize(loadvector,monthhour);

[~,hours] = size(windmonth);
[~,yrW] = size(Wind);
hr = length(irradiance);
yrI = floor(hr/8760);

windmonthMom = zeros(12,4);
irrmonthMom = zeros(12,4);
loadmonthMom = zeros(12,4);


for m = 1:12
    windmonthMom(m,:) =  [mean(windmonth(m,1:monthhour(m)*yrW)), var(windmonth(m,1:monthhour(m)*yrW)),skewness(windmonth(m,1:monthhour(m)*yrW)),kurtosis(windmonth(m,1:monthhour(m)*yrW))];
    irrmonthMom(m,:) = [mean(irrmonth(m,1:monthhour(m)*yrI)), var(irrmonth(m,1:monthhour(m)*yrI)),skewness(irrmonth(m,1:monthhour(m)*yrI)),kurtosis(irrmonth(m,1:monthhour(m)*yrI))];
    loadmonthMom(m,:) = [mean(loadmonth(m,1:monthhour(m))), var(loadmonth(m,1:monthhour(m))), skewness(loadmonth(m,1:monthhour(m))), kurtosis(loadmonth(m,1:monthhour(m)))];
end

%%

nTrees = 10000; %Set number of scenario trees wanted
nScen = 30; %Set number of scenarios wanted
nHours = 72; %Set number of consecutive hours in each sample
totscen = nScen*12;


windmonth_hourMom = zeros(12,24,4);
irrmonth_hourMom = zeros(12,24,4);
loadmonth_hourMom = zeros(12,24,4);

%Calculate moments for each hour in each month in the historic data
for m = 1:12
    for h = 0:23
    windmonth_hourMom(m,h+1,:) =  [mean(windmonth(m,(1:24:(monthhour(m)*yrW))+h)), var(windmonth(m,(1:24:(monthhour(m)*yrW))+h)),skewness(windmonth(m,(1:24:(monthhour(m)*yrW))+h)),kurtosis(windmonth(m,(1:24:(monthhour(m)*yrW))+h))];
    irrmonth_hourMom(m,h+1,:) = [mean(irrmonth(m,(1:24:(monthhour(m)*yrI))+h)), var(irrmonth(m,(1:24:(monthhour(m)*yrI))+h)),skewness(irrmonth(m,(1:24:(monthhour(m)*yrI))+h)),kurtosis(irrmonth(m,(1:24:(monthhour(m)*yrI))+h))];
    loadmonth_hourMom(m,h+1,:) = [mean(loadmonth(m,(1:24:(monthhour(m)))+h)), var(loadmonth(m,(1:24:(monthhour(m)))+h)),skewness(loadmonth(m,(1:24:(monthhour(m)))+h)),kurtosis(loadmonth(m,(1:24:(monthhour(m)))+h))];
    end
end

irrmonth_hourMom(isnan(irrmonth_hourMom))=0;


%%
%Creating scenario trees
scenTrees = zeros(nTrees,nScen*12,3,nHours); %Create scenario tree matrix

%Poulate scenario trees by pseudorandomly sampling from a uniform
%distribution between 1:hours-nHours with replacement, to represent the
%start time for the nHours consecutive hours sampled from each month
for u = 1:nTrees
    for s = 1:nScen
        x = zeros(12);
        for mn = 0:11
            x(mn+1)=randsample(1:24:(monthhour(2)-nHours),1,true);
            yr = randi(yrW)-1;
            for h = 0:nHours-1
                scenTrees(u,s+mn*nScen,1,h+1) = windmonth(mn+1,x(mn+1)+yr*monthhour(mn+1)+h);
                scenTrees(u,s+mn*nScen,2,h+1) = irrmonth(mn+1,x(mn+1)+yr*monthhour(mn+1)+h);
                scenTrees(u,s+mn*nScen,3,h+1) = loadmonth(mn+1,x(mn+1)+h);
            end
        end
    end
end

treeMom = zeros(nTrees,2,4); %Create matrix for moments of wind (1) and irr(2) in each scenario tree
treeMonthMom = zeros(nTrees,2,12,4); %Create matrix for moments of each month 
treeHourMonthMom = zeros(nTrees,2,12,24,4); %Create matrix for moments of each hour in each month

%%
%Choose the scenario trees that are the closest statistical match to
%underlying data, by first calculating moments of complete scentree by
%calculating moments of each scenario and weighting them together, then
%calculating moments of each month in each tree, and weighting these
%together, finally calculating moments of each hour in each month across
%scenarios
for u = 1:nTrees
    for t = 1:2
        %calculating 4 first moments for full tree
        A = squeeze(scenTrees(u,:,t,:));
        meanA = mean(mean(A));
        varA = var(reshape(A,[totscen*nHours,1]));
        skewnessA = skewness(reshape(A,[totscen*nHours,1]));
        kurtosisA = kurtosis(reshape(A,[totscen*nHours,1]));
        treeMom(u,t,:) = [meanA,varA,skewnessA,kurtosisA];
        
        %Calculating 4 first moments for each month in tree
        for m = 0:11
            B = squeeze(scenTrees(u,(1:nScen)+m*nScen,t,1:nHours))';
            meanB = mean(mean(B));
            varB = var(reshape(B,[nHours*nScen,1]));
            skewnessB = skewness(reshape(B,[nHours*nScen,1]));
            kurtosisB = kurtosis(reshape(B,[nHours*nScen,1]));
            treeMonthMom(u,t,m+1,:) = [meanB,varB,skewnessB,kurtosisB];
            
            %Calculating 4 first moments for each hour in each month
            for h = 0:23
            meanC = mean(mean(B((1:24:nHours)+h,:)));
            varC = var(reshape(B((1:24:nHours)+h,:),[floor(nHours/24)*nScen,1]));
            skewnessC = skewness(reshape(B((1:24:nHours)+h,:),[floor(nHours/24)*nScen,1]));
            kurtosisC = kurtosis(reshape(B((1:24:nHours)+h,:),[floor(nHours/24)*nScen,1]));
            treeHourMonthMom(u,t,m+1,h+1,:) = [meanC,varC,skewnessC,kurtosisC];
            end
        end
    end
end

treeHourMonthMom(isnan(treeHourMonthMom))=0;
treeMonthMom(isnan(treeMonthMom))=0;
treeMom(isnan(treeMom)) = 0;

%%
%Compare moments of generated scenario trees and actual historic data

diffMonth = zeros(nTrees,2,12);
diffTree = zeros(nTrees,2);
diffHour = zeros(nTrees,2,12);
diffHourTot = zeros(nTrees,12);

for u=1:nTrees
        for m=1:12
            for h = 1:24
                for v=1:4
                    diffHour(u,1,m) = diffMonth(u,1,m) + abs((windmonth_hourMom(m,h,v)-treeHourMonthMom(u,1,m,h,v))/windmonth_hourMom(m,h,v));
                end
            end
            for v = 1:4
                diffMonth(u,1,m) = diffMonth(u,1,m) + abs((windmonthMom(m,v)-treeMonthMom(u,1,m,v))/windmonthMom(m,v));
                diffTree(u,1) = diffTree(u,1) + abs((windmom(v)-treeMom(u,1,v))/windmom(v));
            end
        end
end



for u=1:nTrees
    for m=1:12
        for h = 1:24
            for v=1:4
                if irrmonth_hourMom(m,h,v) ~= 0
                    diffHour(u,2,m) = diffMonth(u,2,m) + abs((irrmonth_hourMom(m,h,v)-treeHourMonthMom(u,2,m,h,v))/irrmonth_hourMom(m,h,v));
                end
            end
        end
        for v = 1:4
            diffMonth(u,2,m) = diffMonth(u,2,m) + abs((irrmonthMom(m,v)-treeMonthMom(u,2,m,v))/irrmonthMom(m,v));
            diffTree(u,2) = diffTree(u,2) + abs((irrmom(v)-treeMom(u,2,v))/irrmom(v));
        end
    end
end


for u = 1:nTrees
    for m=1:12
        diffHourTot(u,m) = diffHour(u,1,m)+diffHour(u,2,m); 
    end
end

[M,I] = min(diffHourTot(:,:));

optScenTree = zeros(nScen*12,3,nHours);

for m = 0:11
    optScenTree((1:nScen)+m*nScen,:,:) = scenTrees(I(m+1),(1:nScen) + m*nScen,:,:);
end


%%
%Printing chosen scenario trees to excel
    writetoxlsx(optScenTree,22);
    




                
                